from __future__ import nested_scopes

# system modules
import os, sys, urllib, shutil
# local modules
import config, packagelist

if sys.version_info[0] == 2 and sys.version_info[1] >= 2:
    import ftplib
else:
    from Ciphon import ftplib2 as ftplib

__version__ = "$Revision: 1.13 $"

class Installer:
    """Handles installation of packages"""
    def __init__(self, packageName):
        self.__config = config.getConfig()
        self.__packages = packagelist.getPackageList()
        self.__installed = config.getInstalled()
        self.__package = packageName
        self.__version = self.__packages.getVersion(packageName)
        self.__fileName = self.__package + '-' + self.__version + '.tar.gz'

    def install(self):
        if (self.__installed.installed(self.__package) and
            self.__installed.current(self.__package, self.__version)):
            sys.stdout.write('%s is current\n' % self.__package)
            return
        cwd = os.getcwd()
        os.chdir(os.path.expanduser('~/.ciphon/'))

        try:
            getResult = self.__get()
        except:
            raise PackageGetError('Downloading package')

        try:
            self.__unpack()
        except:
            try:
                self.__clean()
            except:
                pass
            raise InstallError('Unpacking package')

        try:
            self.__build()
        except:
            try:
                self.__clean()
            except:
                pass
            raise PackageBuildError('Building package')

        try:
            self.__install()
        except:
            try:
                self.__clean()
            except:
                pass
            raise PackageInstallError('Installing package')

        try:
            self.__installed.update(self.__package, self.__version)
            self.__installed.write()
            self.__clean()
        except:
            raise PackageInstallError('Cleaning package')
        os.chdir(cwd)

    def __get(self):
        """Try to download package"""
        packageLocation = self.__packages.getLocation(self.__package)
        try:
            fileHandle = open(self.__fileName, 'w')
        except OSError:
            PackageGetError("Can't open file for writing")
        downloaded = 0
        try:
            while not downloaded:
                server = self.__config.getServerAddress()
                ftpCommand = "RETR %s/%s/%s" % (server['path'],
                                                packageLocation,
                                                self.__fileName)
                ftpServer = ftplib.FTP()
                ftpServer.connect(server['address'])
                ftpServer.login()
                ftpServer.retrbinary(ftpCommand, fileHandle.write)
                ftpServer.quit()
                break
        except config.ServerError:
            raise PackageGetError("Can't download package from any server!")

        self.__config.resetServerAddress()
        return 0

    def __unpack(self):
        """Unpack a package so that it can be built and installed"""

        returnVal = os.system('tar xvzf %s' % self.__fileName)
        if returnVal == 0:
            return
        else:
            raise PackageBuildError("Can't untar package!")

    def __build(self):
        """Build a package using distutils"""

        presentDir = os.getcwd()
        os.chdir('%s/%s' % (presentDir, self.__fileName[:-7]))

        if self.__config.getParameter('installtype') == 'rpm':
            returnVal = os.system('python2 setup.py bdist_rpm')
        else:
            returnVal = os.system('python2 setup.py build')

        if returnVal == 0:
            os.chdir(presentDir)
            return
        else:
            os.chdir(presentDir)
            raise PackageBuildError("Can't build package!")

    def __install(self):
        """Install a package using disutils"""
        presentDir = os.getcwd()
        os.chdir(os.path.join(presentDir, self.__fileName[:-7], 'dist'))

        if self.__config.getParameter('installtype') == 'rpm':
          returnVal = os.system('rpm -Uvh %s-%s-1.noarch.rpm' % (self.__package,
                                                                 self.__version))
        else:
           returnVal = os.system('python2 setup.py install')

        if returnVal == 0:
            os.chdir(presentDir)
            return
        else:
            os.chdir(presentDir)
            raise PackageInstallError("Can't install package!")

    def __clean(self):
        """Cleans up a package after installation"""
        try:
            os.unlink(self.__fileName)
            shutil.rmtree(self.__fileName[:-7])
        except Error:
            raise PackageInstallError("Can't clean up after package")
        
def main(input):
    """Handle package install requests"""
    try:
        installer = Installer(input[1])
        installer.install()
    except InstallError, x:
        sys.stderr.write('Error installing package: %s\nGiving up...\n' % x);
    except packagelist.NotFound:
        sys.stderr.write('No such package exists!\n');



def installModule(commands, help):
    """Install the install command into the main eval loop"""

    commands['install'] = main
    help['install'] = "install [package]"

# ************* Exceptions thrown by Installer class *****************

class InstallError(Exception):
    """Superclass for all exceptions relating to installation errors"""
    pass

class PackageInstallError(InstallError):
    """Error installing a package"""
    pass

class PackageBuildError(InstallError):
    """ Error building a package"""
    pass

class PackageGetError(InstallError):
    """Error downloading or otherwise obtaining a package"""
    pass
